PRO udsegm,gc,c6,basg,basc,tug,tuc,mask
;
; Segmentation and identification of UDs in a pair of 'gc','c6' frames
; 1. Manual definition of region of interest (DEFROI)
;	This step is done only if parameter 'mask' is not supplied.
;	Selected mask is saved in an IDL save file,
; 2. Strous's segmentation, labelling, and measurement of UDs (LEXTREM_SEG)
; 3. Selection of UDs using size and intensity-excess criteria
; 4. Mutual identification of UDs in the 'gc' and 'c6' images (IDUD.PRO)
;
; INPUTS: gc = working subfield of 'gc' original frame
;	  c6 = working subfield of 'c6' original frame
;	  basg = background surface for 'gc'
;	  basc = background surface for 'c6'
;	  mask = mask (bytarr) of the umbral region;
;		if not available for input, skip this parameter
; OUTPUTS: tug, tuc = 2D LONG arrays/tables for UDs in 'gc','c6';
;	   intensities are in units of Iph=10000
;	  0: x-pos, 1: y-pos, 2: intensity 3: area(pix), 4: backgr.int.
;	  ------------------------------------------------------------
;	   mask (bytarr) of the umbral region saved in file 'mask'
;
; Selection criteria and parameters of segmentation are editable
;	parameters in the program.
; Note: The original images are integer, normalized to Iph=10000
;
; ROUTINES CALLED: tvw, lextrem_seg, sconvol, stroal, objekt2, idud,
;		   ocont, defroi (IDL)
;
on_error,1
if n_params() lt 6 then message,'Params: gc,c6,basg,basc,tug,tuc[,mask]'

; SELECTION CRITERIA -------------------------------------------	

 minsiz = 9	; minimum area of an UD (pix), 9 pix ~ d=0".12
 minexg = 0.05	; minimum difference 'gc' peak-background (Iph=1 units) 
 minexc = 0.05	; minimum difference 'c6' peak-background (Iph=1 units)
 maxdi  = 3	; maximum distance between 'gc' and 'c6' UDs (IDUD.PRO) 

; SEGMENTATION PARAMETERS --------------------------------------

 smg = 4.	; smoothing Gaussian FWHM (pix) for 'gc'
 smc = 3.  ;3.2	; smoothing Gaussian FWHM (pix) for 'c6'
 sh  = 1	; step (pix) parameter for STROAL routine

; --------------------------------------------------------------

si=size(gc)	; image size

; Region of interest (making mask)

if n_params() lt 7 then begin
newsel:
  window,2,xs=si(1),ys=si(2)
  tvscl,gc
  tvw,gc-basg
  print,'Select umbral region manually'
  sbs=DEFROI(si(1),si(2))	; subscripts of region's pixels
  print,'Selection OK ? (y/n)'
  if get_kbrd(1) ne 'y' then goto,newsel
 mask=bytarr(si(1),si(2))
 mask(sbs)=1B
 save,mask,file='mask'
 print,'Mask saved'
endif

; Searching for local maxima (segmentation, masking, labelling,
;	 measurement of position, intensity, and size)

tug=LEXTREM_SEG(gc,smg,1,sh,mask)	; x,y,intensity(10000),size
tuc=LEXTREM_SEG(c6,smc,1,sh,mask)	; LONG arrays (4,n)

; Selection - size criterion

w=where(tug(3,*) ge minsiz,cn1)	; reducing
tug=tug(*,w)
w=where(tuc(3,*) ge minsiz,cn2)
tuc=tuc(*,w)
print,'Size:      selected',cn1,cn2

; Background intensity

ibg=basg(tug(0,*),tug(1,*))	; backgr.intensities "under" maxima
ibc=basc(tuc(0,*),tuc(1,*))

; Selection - peak-background differences criterion

mineg=fix(minexg*10000)		; scaling
minec=fix(minexc*10000)
eg=tug(2,*)-ibg			; peak-background differences
ec=tuc(2,*)-ibc
w=where(eg ge mineg,cn1)		; reducing
tug=tug(*,w)
ibg=ibg(w)
w=where(ec ge minec,cn2)
tuc=tuc(*,w)
ibc=ibc(w)
print,'Intensity: selected',cn1,cn2

; Creating final tables

ibxg=lonarr(1,cn1)
ibxg(0,*)=ibg
ibxc=lonarr(1,cn2)
ibxc(0,*)=ibc
tug=[tug,ibxg]
tuc=[tuc,ibxc]

; Mutual identification of 'gc' and 'c6' UDs

IDUD,tug,tuc,maxdi

window,0,xs=fix(1.5*si(1)+15),ys=fix(1.5*si(2))
ocont,gc-basg,gc-basg,[-501,-500]
ocont,gc-basg,gc-basg,[-501,-500]
oplot,tug(0,*),tug(1,*),psym=1
oplot,tuc(0,*),tuc(1,*),psym=1,col=0
window,2,xs=fix(1.5*si(1)+15),ys=fix(1.5*si(2))
ocont,c6-basc,c6-basc,[-501,-500]
ocont,c6-basc,c6-basc,[-501,-500]
oplot,tuc(0,*),tuc(1,*),psym=1
oplot,tug(0,*),tug(1,*),psym=1,col=0

END


